==> service-tools-0.0.1-20150424.035500-2.jar: bizlogic/tools/http/csrf.clj <==
(ns bizlogic.tools.http.csrf
  (:require [crypto.random :as random]
            [crypto.equality :as crypto]
            [io.pedestal.interceptor :as interceptor
             :refer [definterceptorfn around]]))

;; This is a port of the ring-anti-forgery CSRF protection, with the following
;; differences:
;;  * Optionally include (and handle) a double-submit cookie
;;  * A function to get the current token (to embed in a head/meta tag, html form)
;;    given the context
;;  * CSRF token is in the `request` not in a dynamic var
;;  * Structured for Pedestal interceptor
;;  * No third-party dependency on an HTML templating library

;; af :- anti-forgery
;; this is both the marker and the function (to use with the context)
(def anti-forgery-token ::anti-forgery-token)

(defn existing-af-token [request]
  (get-in request [:session :csrf/anti-forgery-token]))

(defn new-af-token []
  (random/base64 60))

(defn- assoc-session-token [response request af-token]
  (let [existing-af-token (existing-af-token request)]
    (if (= existing-af-token af-token)
      response
      (assoc-in response [:session :csrf/anti-forgery-token] af-token))))

;; This must be run after the session token setting
(defn- assoc-double-submit-cookie [request response]
  (let [token (get-in request [:session :csrf/anti-forgery-token])]
    ;; The token should also be in a cookie for JS (proper double submit)
    (assoc-in response [:cookies "__anti-forgery-token"] token)))

(defn- form-params [request]
  (merge (:form-params request)
         (:multipart-params request)))

(defn- default-req-token-reader [request]
  (or (-> request form-params (get "__anti-forgery-token"))
      (-> request :headers (get "x-csrf-token"))
      (-> request :headers (get "x-xsrf-token"))))

(defn- valid-request? [request req-token-reader]
  (let [req-af-token (req-token-reader request)]
    (if-let [existing-af-token (existing-af-token request)]
      (and req-af-token existing-af-token
        (crypto/eq? req-af-token existing-af-token))
      true)))

(defn- get-request? [{method :request-method :as request}]
  (or (= :head method)
      (= :get method)))

(defn access-denied-response [body]
  {:status 403
   :headers {"Content-Type" "text/html"}
   :body body})

(def denied-msg "<h1>Invalid anti-forgery token</h1>")

(def default-error-response (access-denied-response denied-msg))

(definterceptorfn anti-forgery
  "Interceptor that prevents CSRF attacks. Any POST/PUT/PATCH/DELETE request to
  the handler returned by this function must contain a valid anti-forgery
  token, or else an access-denied response is returned.

  The anti-forgery token can be placed into a HTML page via the
  ::anti-forgery-token within the request, which is bound to a random key
  unique to the current session. By default, the token is expected to be in a
  form field named '__anti-forgery-token', or in the 'X-CSRF-Token' or
  'X-XSRF-Token' headers.

  This behavior can be customized by supplying a map of options:
    :read-token
      a function that takes a request and returns an anti-forgery token, or nil
      if the token does not exist.
    :cookie-token
      a truthy value, if you want a CSRF double-submit cookie set
    :error-response
      the response to return if the anti-forgery token is incorrect or missing.
    :error-handler
      a handler function (passed the context) to call if the anti-forgery
      token is incorrect or missing (intended to return a valid response).

  Only one of :error-response, :error-handler may be specified."
  ([] (anti-forgery {}))
  ([options]
   {:pre [(not (and (:error-response options)
                    (:error-handler options)))]}
   (let [req-token-reader (:req-token-reader options default-req-token-reader)
         cookie-token? (:cookie-token options)
         error-response (:error-response options default-error-response)
         error-handler (:error-handler options (fn [context]
                                                 (assoc-in context [:response] error-response)))]
     (around ::anti-forgery
       (fn [{request :request :as context}]
         (let [req-token (or (req-token-reader request) (new-af-token))]
           (if (and (not (get-request? request))
                    (not (valid-request? request req-token-reader)))
             (error-handler context)
             (assoc-in context [:request anti-forgery-token] req-token))))
       (fn [{response :response req :request :as context}]
         (let [af-token (anti-forgery-token req)]
           (assoc context
                  :response (cond-> (assoc-session-token response req af-token)
                              cookie-token? (#(assoc-double-submit-cookie req %))))))))))

==> service-tools-0.0.1-20150527.145816-3.jar: bizlogic/tools/http/csrf.clj <==
(ns bizlogic.tools.http.csrf
  (:require [crypto.random :as random]
            [crypto.equality :as crypto]
            [io.pedestal.interceptor :as interceptor
             :refer [definterceptorfn around]]))

;; This is a port of the ring-anti-forgery CSRF protection, with the following
;; differences:
;;  * Optionally include (and handle) a double-submit cookie
;;  * A function to get the current token (to embed in a head/meta tag, html form)
;;    given the context
;;  * CSRF token is in the `request` not in a dynamic var
;;  * Structured for Pedestal interceptor
;;  * No third-party dependency on an HTML templating library

;; af :- anti-forgery
;; this is both the marker and the function (to use with the context)
(def anti-forgery-token ::anti-forgery-token)

(defn existing-af-token [request]
  (get-in request [:session :csrf/anti-forgery-token]))

(defn new-af-token []
  (random/base64 60))

(defn- assoc-session-token [response request af-token]
  (let [existing-af-token (existing-af-token request)]
    (if (= existing-af-token af-token)
      response
      (assoc-in response [:session :csrf/anti-forgery-token] af-token))))

;; This must be run after the session token setting
(defn- assoc-double-submit-cookie [request response]
  (let [token (get-in request [:session :csrf/anti-forgery-token])]
    ;; The token should also be in a cookie for JS (proper double submit)
    (assoc-in response [:cookies "__anti-forgery-token"] token)))

(defn- form-params [request]
  (merge (:form-params request)
         (:multipart-params request)))

(defn- default-req-token-reader [request]
  (or (-> request form-params (get "__anti-forgery-token"))
      (-> request :headers (get "x-csrf-token"))
      (-> request :headers (get "x-xsrf-token"))))

(defn- valid-request? [request req-aftoken-reader]
  (let [req-af-token (req-aftoken-reader request)]
    (if-let [existing-af-token (existing-af-token request)]
      (and req-af-token existing-af-token
        (crypto/eq? req-af-token existing-af-token))
      true)))

(defn- get-request? [{method :request-method :as request}]
  (or (= :head method)
      (= :get method)))

(defn access-denied-response [body]
  {:status 403
   :headers {"Content-Type" "text/html"}
   :body body})

(def denied-msg "<h1>Invalid anti-forgery token</h1>")

(def default-error-response (access-denied-response denied-msg))

(definterceptorfn anti-forgery
  "Interceptor that prevents CSRF attacks. Any POST/PUT/PATCH/DELETE request to
  the handler returned by this function must contain a valid anti-forgery
  token, or else an access-denied response is returned.

  The anti-forgery token can be placed into a HTML page via the
  ::anti-forgery-token within the request, which is bound to a random key
  unique to the current session. By default, the token is expected to be in a
  form field named '__anti-forgery-token', or in the 'X-CSRF-Token' or
  'X-XSRF-Token' headers.

  This behavior can be customized by supplying a map of options:
    :read-token
      a function that takes a request and returns an anti-forgery token, or nil
      if the token does not exist.
    :cookie-token
      a truthy value, if you want a CSRF double-submit cookie set
    :error-response
      the response to return if the anti-forgery token is incorrect or missing.
    :error-handler
      a handler function (passed the context) to call if the anti-forgery
      token is incorrect or missing (intended to return a valid response).

  Only one of :error-response, :error-handler may be specified."
  ([] (anti-forgery {}))
  ([options]
   {:pre []}
   (let [req-token-reader (:req-token-reader options default-req-token-reader)
         cookie-token? (:cookie-token options)
         error-response (:error-response options default-error-response)
         error-handler (:error-handler options
                                       (fn [context]
                                         (assoc-in context
                                           [:response] error-response)))]
     (around ::anti-forgery
       (fn [{request :request :as context}]
         (let [req-token (or (req-token-reader request) (new-af-token))]
           (if (and (not (get-request? request))
                    (not (valid-request? request req-token-reader)))
             (error-handler context)
             (assoc-in context [:request anti-forgery-token] req-token))))
       (fn [{response :response req :request :as context}]
         (let [af-token (anti-forgery-token req)]
           (assoc context :response
             (cond-> (assoc-session-token response req af-token)
               cookie-token? (#(assoc-double-submit-cookie req %))))))))))
